package com.unitesk.requality.libreoffice.api;

import java.util.Objects;

public class WrappedResponse<T> {
	private T response;
	private RequestError error;

	public WrappedResponse(T response) {
		this.response = Objects.requireNonNull(response, "response == null");
		this.error = null;
	}

	public WrappedResponse(RequestError error) {
		this.response = null;
		this.error = Objects.requireNonNull(error, "error == null");
	}

	public boolean hasResponse() {
		return response != null;
	}
	
	public T getResponse() {
		if (response == null) {
			throw new IllegalStateException("This WrappedResponse contains no response");
		}
		
		return response;
	}

	public boolean hasError() {
		return error != null;
	}
	
	public RequestError getError() {
		if (error == null) {
			throw new IllegalStateException("This WrappedResponse contains no errors");
		}
		
		return error;
	}
	
	@Override
	public String toString() {
		if (hasError()) {
			return "WrappedResponse[error = " + getError() + "]";
		} else {
			return "WrappedResponse[response = " + getResponse() + "]";
		}
	}
}
