package com.unitesk.requality.libreoffice.api.impls;

import java.io.IOException;
import java.util.UUID;

import com.unitesk.requality.libreoffice.api.APIClient;
import com.unitesk.requality.libreoffice.api.RequestException;
import com.unitesk.requality.libreoffice.api.models.IDMapping;
import com.unitesk.requality.libreoffice.api.models.IDTree;
import com.unitesk.requality.libreoffice.api.requests.QidResolveRequest;
import com.unitesk.requality.libreoffice.api.requests.RequirementExportRequest;
import com.unitesk.requality.libreoffice.api.requests.RequirementImportRequest;
import com.unitesk.requality.libreoffice.api.requests.RequirementsTreeRequest;
import com.unitesk.requality.libreoffice.api.requests.ViewNodeRequest;

public class ProjectAPI {
	private APIClient client;
	private String project;
	
	public ProjectAPI(APIClient client, String project) {
		this.client = client;
		this.project = project;
		
		// TODO: Check that project exists
	}
	
	public String getProject() {
		return project;
	}
	
	public boolean hasNode(String qid) throws IOException {
		try {
			client.execute(new QidResolveRequest(project, qid));
			return true;
		} catch (RequestException e) {
			if (e.getError().getCode().equals("qid_not_found")) {
				return false;
			} else {
				throw e;
			}
		}
	}
	
	public IDMapping resolveQid(String qid) throws IOException {
		return client.execute(new QidResolveRequest(project, qid)).getId();
	}
	
	public IDTree treeRequirements() throws IOException {
		return client.execute(new RequirementsTreeRequest(project));
	}
	
	public String exportRequirement(UUID uuid) throws IOException {
		return client.execute(new RequirementExportRequest(project, uuid)).asText();
	}
	
	public void importRequirement(String html) throws IOException {
		client.execute(new RequirementImportRequest(project, html));
	}
	
	public void viewNode(UUID uuid) throws IOException {
		client.execute(new ViewNodeRequest(project, uuid));
	}
}
