package com.unitesk.requality.libreoffice.api.requests;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.unitesk.requality.libreoffice.api.APIRequest;
import com.unitesk.requality.libreoffice.api.HTTPMethod;
import com.unitesk.requality.libreoffice.api.responses.QidResolveResponse;

public class QidResolveRequest extends APIRequest<QidResolveResponse> {
	private String project;
	private String qid;
	
	public QidResolveRequest(String project, String qid) {
		this.project = project;
		this.qid = qid;
	}

	@JsonIgnore
	public String getProject() {
		return project;
	}

	public String getQid() {
		return qid;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((project == null) ? 0 : project.hashCode());
		result = prime * result + ((qid == null) ? 0 : qid.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		QidResolveRequest other = (QidResolveRequest) obj;
		if (project == null) {
			if (other.project != null)
				return false;
		} else if (!project.equals(other.project))
			return false;
		if (qid == null) {
			if (other.qid != null)
				return false;
		} else if (!qid.equals(other.qid))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return "QidResolveRequest [project=" + project + ", qid=" + qid + "]";
	}

	@Override
	public Class<QidResolveResponse> responseClass() {
		return QidResolveResponse.class;
	}

	@Override
	public String requestURL() {
		return "/projects/" + urlencode(project) + "/qid-resolve";
	}

	@Override
	public HTTPMethod method() {
		return HTTPMethod.POST;
	}
}
