package com.unitesk.requality.libreoffice.comp;

import java.io.IOException;
import java.net.URL;

import com.sun.star.uno.RuntimeException;
import com.unitesk.requality.libreoffice.api.APIClient;
import com.unitesk.requality.libreoffice.api.impls.API;
import com.unitesk.requality.libreoffice.api.impls.ProjectAPI;
import com.unitesk.requality.libreoffice.ui.APISetupDialog;
import com.unitesk.requality.libreoffice.ui.ProjectSelectionDialog;
import com.unitesk.requality.libreoffice.ui.UIUtils;

public class ProjectSelection {
	public static final ProjectSelection instance = new ProjectSelection();
	
	public static class ProjectSelectionCancelled extends RuntimeException {
		private static final long serialVersionUID = 1L;

		public ProjectSelectionCancelled(String message) {
			super(message);
		}
	}
	
	private APIClient client;
	private String currentProject = null;
	
	public API ensureAPI() {
		try {
			if (client == null) {
				client = APIClient.get();
			}
				
			while (true) {
				String message = "Could not connect to Requality. Please check details below and try again.";
				
				try {
					client.testPing();
					break;
				} catch (IOException e) {
				}
				
				UIUtils.prepare();
				APISetupDialog dialog = new APISetupDialog(message, client.getBaseURL(), client.getApiKey());
				if (dialog.isCancelled()) {
					throw new ProjectSelectionCancelled("API client");
				}
				
				client = new APIClient(new URL(dialog.getBaseURL()), dialog.getApiKey());
			}
			
			return new API(client);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	public ProjectAPI ensureProject(boolean force) {
		API api = ensureAPI();
		
		try {
			if (force) {
				currentProject = null;
			}
			
			while (true) {
				if (currentProject != null && api.projectExists(currentProject)) {
					return api.getProject(currentProject);
				}
				
				UIUtils.prepare();
				ProjectSelectionDialog dialog = new ProjectSelectionDialog(api.getProjects());
				if (dialog.isCancelled()) {
					throw new ProjectSelectionCancelled("Project");
				}
				
				currentProject = dialog.getProject();
			}
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	public static ProjectAPI project() {
		return instance.ensureProject(false);
	}
}
