package com.unitesk.requality.libreoffice.comp.refcheck;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

import com.sun.star.text.XTextDocument;
import com.sun.star.uno.XComponentContext;
import com.unitesk.requality.libreoffice.api.impls.ProjectAPI;
import com.unitesk.requality.libreoffice.ui.ErrorSummaryDialog.ErrorSummary;

public class DocumentReferenceChecker {
	private ProjectAPI project;
	private XTextDocument document;
	private ErrorSummary errors;
	
	public DocumentReferenceChecker(ProjectAPI project, XComponentContext context, XTextDocument document) {
		this.project = project;
		this.document = document;
		
		errors = new ErrorSummary();
	}
	
	public void check() throws com.sun.star.uno.Exception, IOException {
		Set<String> seenHeaders = new HashSet<>();
		Set<String> seenTerms = new HashSet<>();
		
		for (ParagraphInfo info: ParagraphInfo.iterate(document)) {
			if (info.isHeading()) {
				if (seenHeaders.contains(info.text)) {
					info.insertBookmark("Duplicated header: " + info.text);
					errors.duplicatedHeaders++;
				}
				
				seenHeaders.add(info.text);
			}
			
			String defTerm = info.getDefTerm();
			if (defTerm != null) {
				if (seenTerms.contains(defTerm)) {
					info.insertBookmark("Duplicated term definition: " + defTerm);
					errors.duplicatedTerms++;
				}
				
				seenTerms.add(defTerm);
			}
		}
		
		for (ParagraphInfo info: ParagraphInfo.iterate(document)) {
			String ref = info.getReference();
			if (ref != null) {
				if (!seenHeaders.contains(ref) && !project.hasNode(ref)) {
					info.insertBookmark("Unresolved reference: " + ref);
					errors.unresolvedReferences++;
				}
			}
			
			String useTerm = info.getUseTerm();
			if (useTerm != null && !seenTerms.contains(useTerm)) {
				info.insertBookmark("Unresolved term: " + useTerm);
				errors.unresolvedTerms++;
			}
		}
	}
	
	public ErrorSummary getErrors() {
		return errors;
	}
}
