package com.unitesk.requality.libreoffice.comp.refcheck;

import static com.sun.star.uno.UnoRuntime.queryInterface;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.sun.star.container.XNamed;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.text.XTextContent;
import com.sun.star.text.XTextDocument;
import com.sun.star.text.XTextRange;

public class ParagraphInfo {
	public static final Pattern HEADING_STYLE_PATTERN	= Pattern.compile("Heading(.+)");
	public static final Pattern ATTRIBUTE_STYLE_PATTERN	= Pattern.compile("(.+)\\.attribute");
	public static final Pattern DEF_TERM_PATTERN		= Pattern.compile("def-term:\\s*(.+)");
	public static final Pattern USE_TERM_PATTERN		= Pattern.compile("use(?:def|-term):\\s*(.+)");
	public static final Pattern REFERENCE_PATTERN		= 
			Pattern.compile("(?:ref|relates):\\s*(.+)", Pattern.CASE_INSENSITIVE);
	
	public XTextDocument document;
	public XTextRange position;
	public String text;
	public String style;
	
	public static ParagraphIterable iterate(XTextDocument doc) {
		return new ParagraphIterable(doc);
	}
	
	public boolean isHeading() {
		return HEADING_STYLE_PATTERN.matcher(style).matches();
	}
	
	public boolean isAttribute() {
		return ATTRIBUTE_STYLE_PATTERN.matcher(style).matches();
	}
	
	public boolean isReference() {
		return isAttribute() && REFERENCE_PATTERN.matcher(text).matches();
	}
	
	private String getAttribute(Pattern p) {
		if (!isAttribute()) {
			return null;
		}
		
		Matcher m = p.matcher(text);
		return m.matches() ? m.group(1) : null;
	}
	
	public String getReference() {
		return getAttribute(REFERENCE_PATTERN);
	}
	
	public String getDefTerm() {
		return getAttribute(DEF_TERM_PATTERN);
	}
	
	public String getUseTerm() {
		return getAttribute(USE_TERM_PATTERN);
	}
	
	public void insertBookmark(String name) throws com.sun.star.uno.Exception {
		XMultiServiceFactory msf = queryInterface(XMultiServiceFactory.class, document);
		Object bookmark = msf.createInstance("com.sun.star.text.Bookmark");
		queryInterface(XNamed.class, bookmark).setName(name);
		document.getText().insertTextContent(position, queryInterface(XTextContent.class, bookmark), true);
	}
}
