package com.unitesk.requality.libreoffice.comp.refcheck;

import static com.sun.star.uno.UnoRuntime.queryInterface;

import java.util.Iterator;

import com.sun.star.beans.XPropertySet;
import com.sun.star.container.XEnumeration;
import com.sun.star.container.XEnumerationAccess;
import com.sun.star.lang.XServiceInfo;
import com.sun.star.text.XTextContent;
import com.sun.star.text.XTextDocument;
import com.sun.star.uno.Exception;
import com.sun.star.uno.RuntimeException;

public class ParagraphIterable implements Iterable<ParagraphInfo> {
	private XTextDocument document;
		
	public ParagraphIterable(XTextDocument document) {
		this.document = document;
	}
	
	private <T> boolean arrayContains(T[] array, T data) {
		for (T x: array) {
			if (x.equals(data)) {
				return true;
			}
		}
		
		return false;
	}

	@Override
	public Iterator<ParagraphInfo> iterator() {
		XEnumeration enumeration = queryInterface(XEnumerationAccess.class, document.getText()).createEnumeration();
	
		return new Iterator<ParagraphInfo>() {
			@Override
			public boolean hasNext() {
				return enumeration.hasMoreElements();
			}

			@Override
			public ParagraphInfo next() {
				Object paragraph;
				try {
					paragraph = enumeration.nextElement();
				} catch (Exception e) {
					throw new RuntimeException("Failed to query next paragraph", e);
				}
				
				XServiceInfo serviceInfo = queryInterface(XServiceInfo.class, paragraph);
				XTextContent paragraphContent = queryInterface(XTextContent.class, paragraph);
				XPropertySet paragraphProps = queryInterface(XPropertySet.class, paragraph);
				
				ParagraphInfo pi = new ParagraphInfo();
				pi.document = document;
				pi.position = paragraphContent.getAnchor();
				pi.text = pi.position.getString();
				pi.style = "";
				
				if (arrayContains(serviceInfo.getSupportedServiceNames(), "com.sun.star.text.Paragraph")) {
					try {
						pi.style = (String) paragraphProps.getPropertyValue("ParaStyleName");
					} catch (Exception e) {
						throw new RuntimeException("Failed to query paragraph properties", e);
					}
				}
				
				return pi;
			}
		};
	}
}
