package com.unitesk.requality.libreoffice.menu;

import com.sun.star.frame.XDesktop;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.lib.uno.helper.WeakBase;
import com.sun.star.task.XJobExecutor;
import com.sun.star.uno.RuntimeException;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.unitesk.requality.libreoffice.comp.ProjectSelection.ProjectSelectionCancelled;
import com.unitesk.requality.libreoffice.ui.ExceptionDialog;

public abstract class BaseHandler extends WeakBase implements XJobExecutor {
	public static final boolean useThreading = false;
	
	protected XComponentContext context;
	protected String triggerParam;
	
	public BaseHandler(XComponentContext context) {
		this.context = context;
	}
	
	protected <T extends XComponent> T getComponent(Class<T> clazz) throws com.sun.star.uno.Exception {
    	XComponentContext remoteContext = context;
    	XMultiComponentFactory remoteServiceManager = remoteContext.getServiceManager(); 
    	Object desktop = remoteServiceManager.createInstanceWithContext("com.sun.star.frame.Desktop", remoteContext);
        XDesktop xDesktop = UnoRuntime.queryInterface(XDesktop.class, desktop);
        XComponent xCurrentComponent = xDesktop.getCurrentComponent();    	
        return UnoRuntime.queryInterface(clazz, xCurrentComponent);
	}
	
	protected abstract void handle() throws Exception;

	protected boolean useThreading() {
		return useThreading;
	}
	
	private void handle0() {
		try {
			handle();
		} catch (ProjectSelectionCancelled e) {
			// ignore
		} catch (Exception e) {
			new ExceptionDialog(e);
			throw new RuntimeException(e);
		}
	}
	
	@Override
	public void trigger(String arg0) {
		this.triggerParam = arg0;
		
		if (useThreading()) {
			Thread t = new Thread() {
				@Override
				public void run() {
					handle0();
				}
			};
			
			t.setDaemon(true);
			t.setName("MenuHandler");
			t.start();
		} else {
			handle0();
		}
	}
}
