package com.unitesk.requality.libreoffice.menu;

import java.util.function.Predicate;
import java.util.regex.Pattern;

import com.sun.star.container.XNameAccess;
import com.sun.star.text.XBookmarksSupplier;
import com.sun.star.text.XTextContent;
import com.sun.star.text.XTextDocument;

import static com.sun.star.uno.UnoRuntime.queryInterface;

import com.sun.star.uno.XComponentContext;
import com.unitesk.requality.libreoffice.api.impls.ProjectAPI;
import com.unitesk.requality.libreoffice.comp.ProjectSelection;
import com.unitesk.requality.libreoffice.comp.refcheck.DocumentReferenceChecker;
import com.unitesk.requality.libreoffice.helper.DialogHelper;
import com.unitesk.requality.libreoffice.registration.ServiceName;
import com.unitesk.requality.libreoffice.ui.ErrorSummaryDialog;
import com.unitesk.requality.libreoffice.ui.ErrorSummaryDialog.ErrorSummary;

@ServiceName("com.unitesk.requality.libreoffice.menu.CheckReferences")
public class CheckReferencesHandler extends BaseHandler {
	public static final Pattern HEADING_STYLE_PATTERN	= Pattern.compile("Heading(.+)");
	public static final Pattern ATTRIBUTE_STYLE_PATTERN	= Pattern.compile("(.+)\\.attribute");
	public static final Pattern OUR_BOOKMARK_PATTERN	= 
			Pattern.compile("(Unresolved (reference|term)|Duplicated (header|term definition)):\\s*.+");
	
	public static final Pattern REFERENCE_PATTERN		= 
			Pattern.compile("(ref|relates): (.+)", Pattern.CASE_INSENSITIVE);
	
	public CheckReferencesHandler(XComponentContext context) {
		super(context);
	}

	public static int removeBookmarks(XTextDocument doc, Predicate<String> namePred) {
		XNameAccess bookmarks = queryInterface(XBookmarksSupplier.class, doc).getBookmarks();
		String[] elementNames = bookmarks.getElementNames();
		int count = 0;
		
		for (String s: elementNames) {
			if (namePred.test(s)) 
				try {
					XTextContent bookmark = queryInterface(XTextContent.class, bookmarks.getByName(s));
					doc.getText().removeTextContent(bookmark);
					count++;
				} catch (com.sun.star.uno.Exception e) {
					System.err.println("Failed to remove bookmark '" + s + "'");
					e.printStackTrace();
				}
		}
		
		return count;
	}
	
	public static int removeOurBookmarks(XTextDocument doc) {
		return removeBookmarks(doc, x -> OUR_BOOKMARK_PATTERN.matcher(x).matches());
	}
	
	@Override
	protected void handle() throws Exception {
		ProjectAPI project = ProjectSelection.project();
		XTextDocument document = getComponent(XTextDocument.class);

		removeOurBookmarks(document);
		
		DocumentReferenceChecker rchecker = new DocumentReferenceChecker(project, context, document);
		rchecker.check();
		ErrorSummary errors = rchecker.getErrors();
		
		if (errors.totalErrors() == 0) DialogHelper.showInfoMessage(context, null, "No errors were found.");
		else new ErrorSummaryDialog(errors);
	}
}
