package com.unitesk.requality.libreoffice.menu;

import com.sun.star.beans.PropertyAttribute;
import com.sun.star.beans.XPropertySet;
import com.sun.star.container.XIndexAccess;
import com.sun.star.container.XNameContainer;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.style.XStyle;
import com.sun.star.style.XStyleFamiliesSupplier;
import com.sun.star.text.XTextCursor;
import com.sun.star.text.XTextDocument;
import com.sun.star.text.XTextRange;
import com.sun.star.uno.Any;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.uno.XInterface;
import static com.sun.star.uno.UnoRuntime.queryInterface;
import com.unitesk.requality.libreoffice.helper.DialogHelper;
import com.unitesk.requality.libreoffice.registration.ServiceName;

@ServiceName("com.unitesk.requality.libreoffice.menu.MarkObject")
public class MarkObjectHandler extends BaseHandler {
	private static enum StyleMeta {
		REQUIREMENT("ffefff"),
		TODO("ffffef"),
		ATTRIBUTE("eff6ff"),
		TEXTNODE("ccffcc"),
		TEXTNODE_DEF("ccffcc"),
		TEXTNODE_JUST("daebdc"),
		TEXTNODE_NOTE("99ff66"),
		REVIEW("ffc04d");
		
		private String background;
		
		private StyleMeta(String background) {
			this.background = background;
		}
	}
	
	public MarkObjectHandler(XComponentContext context) {
		super(context);
	}

	@Override
	protected void handle() throws Exception {
		XTextDocument doc = getComponent(XTextDocument.class);
		XMultiServiceFactory serviceFactory = queryInterface(XMultiServiceFactory.class, doc);
		XStyleFamiliesSupplier styleSupplier = queryInterface(XStyleFamiliesSupplier.class, doc);
		XNameContainer styles = queryInterface(XNameContainer.class, 
				styleSupplier.getStyleFamilies().getByName("ParagraphStyles"));
		
		String styleName = null,bases=null;
		for (String s: styles.getElementNames()) {
		    Object o = styles.getByName(s);
		    XStyle xs = null;
		    com.sun.star.uno.Any an = null;
		    if(o instanceof XStyle)
		      xs = (XStyle) o;
		    else{
		      an = (Any) o;
		      xs = (XStyle) an.getObject();
		    }
			if (s.endsWith("." + triggerParam)) {
				styleName = s;
				break;
			}
			if(xs!=null){
			  if(xs.getParentStyle()!=null&&xs.getParentStyle().toLowerCase().equals("standard") && 
			      (bases == null || xs.getName().toLowerCase().equals("text body")))
			    bases = xs.getName();
			}
		}
		
		if (styleName == null) {
			styleName = bases+"." + triggerParam;
			StyleMeta meta = StyleMeta.valueOf(triggerParam.toUpperCase().replaceAll("-", "_"));
			
			
			XInterface paraStyleObj = (XInterface) serviceFactory.createInstance("com.sun.star.style.ParagraphStyle");

			XStyle sx = (XStyle) queryInterface(XStyle.class, paraStyleObj);
			sx.setParentStyle(bases);
			
			XPropertySet paraStyle = queryInterface(XPropertySet.class, paraStyleObj);
			
			int margin = 100;
			paraStyle.setPropertyValue("ParaBackColor", Integer.decode("#" + meta.background));
			paraStyle.setPropertyValue("ParaLeftMargin", margin);
			paraStyle.setPropertyValue("ParaRightMargin", margin);
			paraStyle.setPropertyValue("ParaTopMargin", margin);
			paraStyle.setPropertyValue("ParaBottomMargin", margin);
			
			styles.insertByName(styleName, paraStyleObj);
		}
		
		XIndexAccess access = UnoRuntime.queryInterface(XIndexAccess.class, doc.getCurrentSelection());
		XTextRange range = UnoRuntime.queryInterface(XTextRange.class, access.getByIndex(0));
		XTextCursor cursor = doc.getText().createTextCursorByRange(range);
		XPropertySet cursorProperties = queryInterface(XPropertySet.class, cursor);
		cursorProperties.setPropertyValue("ParaStyleName", styleName);
	}
}
