package com.unitesk.requality.libreoffice.menu;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.URISyntaxException;
import java.nio.charset.StandardCharsets;
import java.util.UUID;

import com.sun.star.beans.PropertyValue;
import com.sun.star.frame.XComponentLoader;
import com.sun.star.frame.XDesktop;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.task.XJobExecutor;
import com.sun.star.text.XTextDocument;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.unitesk.requality.libreoffice.Utils;
import com.unitesk.requality.libreoffice.api.impls.ProjectAPI;
import com.unitesk.requality.libreoffice.comp.ProjectSelection;
import com.unitesk.requality.libreoffice.registration.ServiceName;
import com.unitesk.requality.libreoffice.ui.TreeSelectDialog;

@ServiceName("com.unitesk.requality.libreoffice.menu.OpenDocument")
public class OpenDocumentHandler extends BaseHandler implements XJobExecutor {
	public OpenDocumentHandler(XComponentContext context) {
		super(context);
	}

	private XTextDocument createDocument(File f, UUID reqSource) throws com.sun.star.uno.Exception, URISyntaxException {
		XComponentContext remoteContext = context;
    	XMultiComponentFactory remoteServiceManager = remoteContext.getServiceManager(); 
    	Object desktop = remoteServiceManager.createInstanceWithContext("com.sun.star.frame.Desktop", remoteContext);
        XDesktop xDesktop = UnoRuntime.queryInterface(XDesktop.class, desktop);
	
        XComponentLoader loader = UnoRuntime.queryInterface(XComponentLoader.class, xDesktop);
        Object rawDoc = loader.loadComponentFromURL(Utils.fileToUri(f), "_blank", 0, new PropertyValue[0]);
        XTextDocument doc = UnoRuntime.queryInterface(XTextDocument.class, rawDoc);
        
        return doc;
	}
	
	@Override
	protected void handle() throws Exception {
		ProjectAPI api = ProjectSelection.project();
		TreeSelectDialog select = new TreeSelectDialog(api.treeRequirements());
		if (select.isCancelled()) {
			return;
		}
		
		UUID selectedNode = select.getSelectedNode().getUuid();
		
		File tmpFile = File.createTempFile("requality_", ".html");
		try (Writer w = new OutputStreamWriter(new FileOutputStream(tmpFile), StandardCharsets.UTF_8)) {
			w.write(api.exportRequirement(selectedNode));
			w.close();
		}
		
		createDocument(tmpFile, select.getSelectedNode().getUuid());
		if (!tmpFile.delete())
			tmpFile.deleteOnExit();
	}
}
