package com.unitesk.requality.libreoffice.menu;

import java.io.File;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import com.sun.star.beans.PropertyState;
import com.sun.star.beans.PropertyValue;
import com.sun.star.document.FilterOptionsRequest;
import com.sun.star.frame.XStorable;
import com.sun.star.frame.XStorable2;
import com.sun.star.text.XTextDocument;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.unitesk.requality.libreoffice.Utils;
import com.unitesk.requality.libreoffice.api.impls.ProjectAPI;
import com.unitesk.requality.libreoffice.comp.ProjectSelection;
import com.unitesk.requality.libreoffice.helper.DialogHelper;
import com.unitesk.requality.libreoffice.registration.ServiceName;

@ServiceName("com.unitesk.requality.libreoffice.menu.SaveDocument")
public class SaveDocumentHandler extends BaseHandler {

	public SaveDocumentHandler(XComponentContext context) {
		super(context);
	}

	@Override
	protected void handle() throws Exception {
		ProjectAPI api = ProjectSelection.project();
		XTextDocument doc = getComponent(XTextDocument.class);
		XStorable storable = UnoRuntime.queryInterface(XStorable.class, doc);
		
		File tmpFile = File.createTempFile("requality_", ".html");
		
		PropertyValue[] values = new PropertyValue[1];
		values[0] = new PropertyValue();		
		values[0].Name = "FilterName";
		values[0].Value =  "HTML (StarWriter)";
		storable.storeAsURL(Utils.fileToUri(tmpFile), values);
		
		System.out.println("Saved as " + tmpFile.getCanonicalPath());
		
		String html = new String(Files.readAllBytes(tmpFile.toPath()));
		api.importRequirement(html);
		
		if (!tmpFile.delete())
			tmpFile.deleteOnExit();
		
		DialogHelper.showInfoMessage(context, null, "Document has been successfully committed to Requality project '" + 
													api.getProject()+"'");
	}
}
