package com.unitesk.requality.libreoffice.menu;

import com.sun.star.container.XIndexAccess;
import com.sun.star.text.XTextDocument;
import com.sun.star.text.XTextRange;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.unitesk.requality.libreoffice.api.RequestException;
import com.unitesk.requality.libreoffice.api.impls.ProjectAPI;
import com.unitesk.requality.libreoffice.api.models.IDMapping;
import com.unitesk.requality.libreoffice.comp.ProjectSelection;
import com.unitesk.requality.libreoffice.helper.DialogHelper;
import com.unitesk.requality.libreoffice.registration.ServiceName;

@ServiceName("com.unitesk.requality.libreoffice.menu.ViewNode")
public class ViewNodeHandler extends BaseHandler {
	public ViewNodeHandler(XComponentContext context) {
		super(context);
	}

	@Override
	protected void handle() throws Exception {
		ProjectAPI project = ProjectSelection.project();
		
		XTextDocument doc = getComponent(XTextDocument.class);
		XIndexAccess access = UnoRuntime.queryInterface(XIndexAccess.class, doc.getCurrentSelection());
		XTextRange range = UnoRuntime.queryInterface(XTextRange.class, access.getByIndex(0));
		
		String str = range.getString();
		if (str == null || str.isEmpty()) {
			DialogHelper.showWarningMessage(context, null, "Nothing is selected");
			return;
		}
		
		str = str.substring(0, Math.min(4096, str.length()));
		
		try {
			IDMapping ids = project.resolveQid(str);
			project.viewNode(ids.getUuid());
		} catch (RequestException e) {
			if (e.getError().getCode().equals("qid_not_found")) {
				DialogHelper.showErrorMessage(context, null, "Selected string '" + str + "' is not a reference");
			} else {
				throw e;
			}
		}
	}
}
