package com.unitesk.requality.libreoffice.registration;

import java.util.HashMap;
import java.util.Map;

import com.sun.star.lang.IllegalArgumentException;
import com.sun.star.lang.XSingleComponentFactory;
import com.sun.star.lib.uno.helper.Factory;
import com.sun.star.registry.XRegistryKey;
import com.unitesk.requality.libreoffice.menu.CheckReferencesHandler;
import com.unitesk.requality.libreoffice.menu.ClearBookmarksHandler;
import com.unitesk.requality.libreoffice.menu.MarkObjectHandler;
import com.unitesk.requality.libreoffice.menu.OpenDocumentHandler;
import com.unitesk.requality.libreoffice.menu.ProjectSelectHandler;
import com.unitesk.requality.libreoffice.menu.SaveDocumentHandler;
import com.unitesk.requality.libreoffice.menu.ViewNodeHandler;

public class RegistrationHandler {
	public static final Class<?>[] REGISTERED_CLASSES = new Class<?>[] {
		ProjectSelectHandler.class, CheckReferencesHandler.class, ClearBookmarksHandler.class,
		OpenDocumentHandler.class, SaveDocumentHandler.class, ViewNodeHandler.class, MarkObjectHandler.class
	};
	
	private static Map<String, Class<?>> classesByName;
	private static Map<Class<?>, String[]> services;
	
	public static XSingleComponentFactory __getComponentFactory(String sImplementationName) {
		Class<?> cls = classesByName.get(sImplementationName);
		if (cls == null) {
			return null;
		}
		
		return Factory.createComponentFactory(cls, services.get(cls));
	}
	
	public static boolean __writeRegistryServiceInfo(XRegistryKey xRegistryKey ) {
		boolean success = true;
		
		for (Class<?> c: REGISTERED_CLASSES) {
			success &= Factory.writeRegistryServiceInfo(c.getCanonicalName(), services.get(c), xRegistryKey);
		}
		
		return success;
	}
	
	static {
		classesByName = new HashMap<>();
		services = new HashMap<>();
		
		for (Class<?> c: REGISTERED_CLASSES) {
			classesByName.put(c.getCanonicalName(), c);
			
			ServiceName[] names = c.getAnnotationsByType(ServiceName.class);
			if (names.length == 0) {
				throw new IllegalArgumentException("Class " + c.getCanonicalName() + " has no @ServiceName annotations");
			}
			
			String[] strNames = new String[names.length];
			for (int i = 0; i < names.length; i++) {
				strNames[i] = names[i].value();
			}
			
			services.put(c, strNames);
		}
	}
}
