package com.unitesk.requality.libreoffice.ui;

import javax.swing.JDialog;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JPanel;

import java.awt.Insets;

import javax.swing.JButton;
import javax.swing.JLabel;

public class ErrorSummaryDialog extends JDialog {
	private static final long serialVersionUID = 1L;

	public static class ErrorSummary {
		public int duplicatedHeaders = 0;
		public int duplicatedTerms = 0;
		public int unresolvedReferences = 0;
		public int unresolvedTerms = 0;
		
		public int totalErrors() {
			return duplicatedHeaders + duplicatedTerms + unresolvedReferences + unresolvedTerms;
		}
	}
	
	private int errorCount = 0;
	private JPanel errorPanel;
	
	public ErrorSummaryDialog(ErrorSummary errors) {
		super((JDialog) null, "Errors were found", ModalityType.APPLICATION_MODAL);
		setSize(473, 191);
		
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[]{0, 0};
		gridBagLayout.rowHeights = new int[]{0, 0, 0};
		gridBagLayout.columnWeights = new double[]{0.0, 1.0};
		gridBagLayout.rowWeights = new double[]{0.0, 1.0, 0.0};
		getContentPane().setLayout(gridBagLayout);
		
		JLabel lblErrors = new JLabel("Following errors were found in this document:");
		GridBagConstraints gbc_lblErrors = new GridBagConstraints();
		gbc_lblErrors.anchor = GridBagConstraints.WEST;
		gbc_lblErrors.insets = new Insets(20, 0, 10, 20);
		gbc_lblErrors.gridx = 1;
		gbc_lblErrors.gridy = 0;
		getContentPane().add(lblErrors, gbc_lblErrors);
		
		ImagePanel imagePanel = new ImagePanel("/com/unitesk/requality/error_icon.png");
		GridBagConstraints gbc_imagePanel = new GridBagConstraints();
		gbc_imagePanel.gridheight = 3;
		gbc_imagePanel.insets = new Insets(20, 20, 20, 10);
		gbc_imagePanel.fill = GridBagConstraints.BOTH;
		gbc_imagePanel.gridx = 0;
		gbc_imagePanel.gridy = 0;
		getContentPane().add(imagePanel, gbc_imagePanel);
		
		errorPanel = new JPanel();
		GridBagConstraints gbc_errorPanel = new GridBagConstraints();
		gbc_errorPanel.insets = new Insets(0, 0, 10, 20);
		gbc_errorPanel.fill = GridBagConstraints.BOTH;
		gbc_errorPanel.gridx = 1;
		gbc_errorPanel.gridy = 1;
		getContentPane().add(errorPanel, gbc_errorPanel);
		GridBagLayout gbl_errorPanel = new GridBagLayout();
		gbl_errorPanel.columnWeights = new double[]{1.0, 0.0};
		errorPanel.setLayout(gbl_errorPanel);
		
		JButton btnOk = new JButton("OK");
		btnOk.addActionListener(e -> dispose());
		
		GridBagConstraints gbc_btnOk = new GridBagConstraints();
		gbc_btnOk.insets = new Insets(0, 0, 20, 20);
		gbc_btnOk.anchor = GridBagConstraints.EAST;
		gbc_btnOk.gridwidth = 2;
		gbc_btnOk.gridx = 0;
		gbc_btnOk.gridy = 2;
		getContentPane().add(btnOk, gbc_btnOk);
		
		addError("Unresolved references", errors.unresolvedReferences);
		addError("Unresolved terms", errors.unresolvedTerms);
		addError("Duplicated headers", errors.duplicatedHeaders);
		addError("Duplicated terms", errors.duplicatedTerms);
		
		gbl_errorPanel.rowHeights = new int[errorCount + 1];
		gbl_errorPanel.rowWeights = new double[errorCount + 1];
		gbl_errorPanel.rowWeights[errorCount] = Double.MIN_VALUE;
		
		setLocationRelativeTo(null);
		setVisible(true);
	}
	
	private void addError(String desc, int count) {
		if (count == 0) {
			return;
		}
		
		JLabel descLabel = new JLabel(desc);
		GridBagConstraints gbc_desc = new GridBagConstraints();
		gbc_desc.fill = GridBagConstraints.HORIZONTAL;
		gbc_desc.gridx = 0;
		gbc_desc.gridy = errorCount;
		errorPanel.add(descLabel, gbc_desc);
		
		JLabel countLabel = new JLabel(String.format("%d error%s", count, count == 1 ? "" : "s"));
		GridBagConstraints gbc_count = new GridBagConstraints();
		gbc_count.fill = GridBagConstraints.HORIZONTAL;
		gbc_count.gridx = 1;
		gbc_count.gridy = errorCount;
		errorPanel.add(countLabel, gbc_count);
		
		errorCount++;
	}
}
