package com.unitesk.requality.libreoffice.ui;

import javax.swing.JDialog;

import com.unitesk.requality.libreoffice.api.models.IDMapping;
import com.unitesk.requality.libreoffice.api.models.IDTree;

import java.awt.GridBagLayout;

import javax.swing.JLabel;

import java.awt.GridBagConstraints;

import javax.swing.JPanel;

import java.awt.Insets;

import javax.swing.JTree;
import javax.swing.JButton;

import java.awt.FlowLayout;
import javax.swing.JScrollPane;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

public class TreeSelectDialog extends JDialog {
	private static final long serialVersionUID = 1L;
	
	private boolean cancelled = true;
	private JTree tree;
	private JLabel lblCurrent;
	private JButton btnOk;
	
	private class NodeObject {
		public String name;
		public IDMapping id;
		
		public NodeObject(IDMapping id) {
			this.id = id;
			
			name = id.getUv().isEmpty() ? id.getQid() : id.getUv();
			int lastSlash = name.lastIndexOf('/');
			if (lastSlash != -1) name = name.substring(lastSlash + 1);
		}
		
		public String toString() {
			return name;
		}
	}
	
	public TreeSelectDialog(IDTree contents) {
		super((JDialog) null, "Select subtree", ModalityType.APPLICATION_MODAL);
		setSize(564, 372);
		
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[]{0, 0};
		gridBagLayout.rowHeights = new int[]{0, 0, 0, 0, 0};
		gridBagLayout.columnWeights = new double[]{1.0, Double.MIN_VALUE};
		gridBagLayout.rowWeights = new double[]{0.0, 1.0, 0.0, 0.0, Double.MIN_VALUE};
		getContentPane().setLayout(gridBagLayout);
		
		JLabel lblNewLabel = new JLabel("Please select requirement subtree:");
		GridBagConstraints gbc_lblNewLabel = new GridBagConstraints();
		gbc_lblNewLabel.anchor = GridBagConstraints.WEST;
		gbc_lblNewLabel.insets = new Insets(10, 10, 5, 10);
		gbc_lblNewLabel.gridx = 0;
		gbc_lblNewLabel.gridy = 0;
		getContentPane().add(lblNewLabel, gbc_lblNewLabel);
		
		JScrollPane scrollPane = new JScrollPane();
		GridBagConstraints gbc_scrollPane = new GridBagConstraints();
		gbc_scrollPane.fill = GridBagConstraints.BOTH;
		gbc_scrollPane.insets = new Insets(0, 10, 5, 10);
		gbc_scrollPane.gridx = 0;
		gbc_scrollPane.gridy = 1;
		getContentPane().add(scrollPane, gbc_scrollPane);
		
		tree = new JTree();
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.addTreeSelectionListener(new TreeSelectionListener() {
			@Override
			public void valueChanged(TreeSelectionEvent e) {
				NodeObject n = getSelection();
				if (n == null) {
					lblCurrent.setText("");
					btnOk.setEnabled(false);
				} else {
					lblCurrent.setText(n.id.getQid());
					btnOk.setEnabled(true);
				}
			}
		});
		scrollPane.setViewportView(tree);
		
		lblCurrent = new JLabel("");
		GridBagConstraints gbc_lblCurrent = new GridBagConstraints();
		gbc_lblCurrent.anchor = GridBagConstraints.WEST;
		gbc_lblCurrent.insets = new Insets(0, 10, 5, 10);
		gbc_lblCurrent.gridx = 0;
		gbc_lblCurrent.gridy = 2;
		getContentPane().add(lblCurrent, gbc_lblCurrent);
		
		JPanel panel = new JPanel();
		FlowLayout flowLayout = (FlowLayout) panel.getLayout();
		flowLayout.setAlignment(FlowLayout.RIGHT);
		GridBagConstraints gbc_panel = new GridBagConstraints();
		gbc_panel.insets = new Insets(0, 10, 10, 10);
		gbc_panel.fill = GridBagConstraints.BOTH;
		gbc_panel.gridx = 0;
		gbc_panel.gridy = 3;
		getContentPane().add(panel, gbc_panel);
		
		JButton btnCancel = new JButton("Cancel");
		btnCancel.addActionListener(e -> dispose());
		
		panel.add(btnCancel);
		
		btnOk = new JButton("OK");
		btnOk.setEnabled(false);
		btnOk.addActionListener(e -> { cancelled = false; dispose(); });
		panel.add(btnOk);
		
		fillTree(contents);
		
		setLocationRelativeTo(null);
		setVisible(true);
	}
	
	public boolean isCancelled() {
		return cancelled;
	}
	
	public IDMapping getSelectedNode() {
		NodeObject sel = getSelection();
		return (sel == null) ? null : sel.id;
	}
	
	private NodeObject getSelection() {
		DefaultMutableTreeNode n = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
		return (n == null) ? null : (NodeObject) n.getUserObject();
	}
	
	private void fillTreeNode(IDTree list, DefaultMutableTreeNode node) {
		node.setUserObject(new NodeObject(list.getCurrent()));
		for (IDTree child: list.getChildren()) {
			DefaultMutableTreeNode childNode = new DefaultMutableTreeNode();
			node.add(childNode);
			fillTreeNode(child, childNode);
		}
	}
	
	private void fillTree(IDTree list) {
		DefaultMutableTreeNode root = new DefaultMutableTreeNode("<root>");
		fillTreeNode(list, root);
		tree.setModel(new DefaultTreeModel(root, false));
	}
}
